@@ -78,7 +78,7 @@ def test_error_response(mock):
     mock.return_value = TestResponse(
         status_code=500, data={
             'errors': {
-                'detail': 'Internal server error'}})
+                'details': 'Internal server error'}})
     with assert_raises(SanError):
         san.get(
             "prices/santiment_usd",
@@ -466,3 +466,37 @@ def test_invalid_request():
 def test_invalid_method():
     with assert_raises(SanError):
         san.get("invalid_method/slug")
+
+
+def test_rate_limits():
+    exception = SanError('API Rate Limit Reached. Try again in 366 seconds(7 minutes)')
+    
+    assert san.is_rate_limit_exception(exception)
+    assert san.rate_limit_time_left(exception) == 366
+
+
+def test_api_calls_made_exception_without_key():
+    with assert_raises(SanError):
+        san.api_calls_made()
+
+
+def test_transform_arg_transform_given():
+    kwargs = {
+        'transform': {'type': 'moving_average', 'moving_average_base': 100},
+        'from_date':'utc_now-30d',
+        'to_date': 'utc_now',
+        'slug': 'bitcoin',
+        'interval': '1d'
+    }
+    assert san.sanbase_graphql._transform_arg_helper(kwargs) ==\
+        'transform:{type: \"moving_average\"\nmoving_average_base: 100\n}'
+
+
+def test_transform_arg_no_transform_given():
+    kwargs = {
+        'from_date':'utc_now-30d',
+        'to_date': 'utc_now',
+        'slug': 'bitcoin',
+        'interval': '1d'
+    }
+    assert san.sanbase_graphql._transform_arg_helper(kwargs) == ''
